package com.jolira.jagent;

import static java.util.logging.Level.FINE;
import static org.objectweb.asm.Opcodes.ARETURN;
import static org.objectweb.asm.Opcodes.BASTORE;
import static org.objectweb.asm.Opcodes.BIPUSH;
import static org.objectweb.asm.Opcodes.DUP;
import static org.objectweb.asm.Opcodes.ICONST_0;
import static org.objectweb.asm.Opcodes.ICONST_1;
import static org.objectweb.asm.Opcodes.ICONST_3;
import static org.objectweb.asm.Opcodes.ICONST_4;
import static org.objectweb.asm.Opcodes.INVOKESTATIC;
import static org.objectweb.asm.Opcodes.NEWARRAY;
import static org.objectweb.asm.Opcodes.T_BYTE;

import java.util.logging.Logger;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.MethodVisitor;

final class InetAddressDecorator extends ClassAdapter {
    private final static Logger LOG = Logger
            .getLogger(InetAddressDecorator.class.getName());

    InetAddressDecorator(final ClassVisitor writer) {
        super(writer);
    }

    /**
     * @see ClassAdapter#visitMethod(int, String, String, String, String[])
     */
    @Override
    public MethodVisitor visitMethod(final int access, final String name,
            final String desc, final String signature, final String[] exceptions) {
        if (!"getLocalHost".equals(name)) {
            if (LOG.isLoggable(FINE)) {
                LOG.fine("Skipping method " + name);
            }

            return super.visitMethod(access, name, desc, signature, exceptions);
        }

        LOG.info("Instrumenting method " + name);

        final MethodVisitor mv = cv.visitMethod(access, name, desc, signature,
                exceptions);

        mv.visitCode();

        mv.visitInsn(ICONST_4);
        mv.visitIntInsn(NEWARRAY, T_BYTE);
        mv.visitInsn(DUP);
        mv.visitInsn(ICONST_0);
        mv.visitIntInsn(BIPUSH, 127);
        mv.visitInsn(BASTORE);
        mv.visitInsn(DUP);
        mv.visitInsn(ICONST_3);
        mv.visitInsn(ICONST_1);
        mv.visitInsn(BASTORE);
        mv.visitMethodInsn(INVOKESTATIC, "java/net/InetAddress",
                "getByAddress", "([B)Ljava/net/InetAddress;");
        mv.visitInsn(ARETURN);
        mv.visitMaxs(4, 0);
        mv.visitEnd();

        return mv;
    }
}
